@@ -10,7 +10,6 @@ import os
 import sys
 import inspect
 from functools import partial
-from copy import deepcopy
 
 def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer, warmup_rounds, global_rounds, local_steps, reg = None, device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')):
     train_splited, test_splited, split_para = dataset_splited
@@ -20,7 +19,10 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
     # initialize
     server = server_class(device)
     server.assign_model(model())
-    server.model_g = deepcopy(model_g)
+
+    server.model_g = model()
+    server.model_g.load_state_dict(model_g.state_dict())
+    server.model_g.to(device)
 
     nodes = [node(i, device) for i in range(num_nodes)]
 
@@ -31,7 +33,10 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
         nodes[i].assign_test(DataLoader(test_splited[i], batch_size=batch_size, shuffle=False))
         # objective
         nodes[i].assign_objective(objective())
-        nodes[i].model_g = deepcopy(model_g)
+
+        nodes[i].model_g = model()
+        nodes[i].model_g.load_state_dict(model_g.state_dict())
+        nodes[i].model_g.to(device)
 
     del train_splited, test_splited
 
@@ -52,6 +57,7 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
                 if nodes[i].local_train_acc(cluster_models[m])<=nodes[i].local_train_acc(cluster_models[k]):
                     m = k
             assignment[m].append(i)
+            nodes[i].label = m
             nodes[i].assign_model(cluster_models[m])
             nodes[i].assign_optim({'local': optimizer(nodes[i].model.parameters()),\
                 'global': optimizer(nodes[i].model_g.parameters()),\
@@ -63,13 +69,13 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
         print([len(assignment[i]) for i in range(len(assignment))])
 
         # local update
-        # for j in range(num_nodes):
-        #     nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_res, optimizer = nodes[j].optim['local'], \
-        #         model_opt = nodes[j].model, model_fix = nodes[j].model_g))
-        
         for j in range(num_nodes):
-            nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_res, optimizer = nodes[j].optim['all'], \
+            nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_res, optimizer = nodes[j].optim['local'], \
                 model_opt = nodes[j].model, model_fix = nodes[j].model_g))
+        
+        # for j in range(num_nodes):
+        #     nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_res, optimizer = nodes[j].optim['all'], \
+        #         model_opt = nodes[j].model, model_fix = nodes[j].model_g))
 
         # server aggregation and distribution by cluster
         for k in range(K):
@@ -78,9 +84,9 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
                 server.distribute(nodes, assignment[k])
                 cluster_models[k].load_state_dict(server.model.state_dict())
         
-        # for j in range(num_nodes):
-        #     nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_res, optimizer = nodes[j].optim['global'],\
-        #         model_opt = nodes[j].model_g, model_fix = cluster_models[[l for l in range(K) if j in assignment[l]][0]]))
+        for j in range(num_nodes):
+            nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_res, optimizer = nodes[j].optim['global'],\
+                model_opt = nodes[j].model_g, model_fix = cluster_models[nodes[j].label]))
             
         # aggregate model_g
         server.aggregate_model_g(nodes, list(range(num_nodes)))
