@@ -1,4 +1,3 @@
-from copy import deepcopy
 import torch
 from torch import linalg as LA
 from fedbase.utils.model_utils import save_checkpoint, load_checkpoint
@@ -30,7 +29,6 @@ class node():
             self.model.load_state_dict(model.state_dict())
         except:
             self.model = model
-        # self.model = deepcopy(model)
         self.model.to(self.device)
         # try:
         #     self.model = torch.compile(self.model)
@@ -80,12 +78,13 @@ class node():
         labels = labels.to(self.device, dtype = torch.long)
         # print(labels)
         # zero the parameter gradients
-        self.model.zero_grad(set_to_none=True)
+        # self.model.zero_grad(set_to_none=True)
+        self.optim.zero_grad()
         # forward + backward + optimize
         outputs = self.model(inputs)
         # optim
-        self.loss = self.objective(outputs, F.one_hot(labels, outputs.shape[1]).float())
-        self.loss.backward()
+        loss = self.objective(outputs, F.one_hot(labels, outputs.shape[1]).float())
+        loss.backward()
 
         # calculate accumulate gradients
         # grads = torch.tensor([])
@@ -93,7 +92,7 @@ class node():
         #     # param.grad = torch.tensor(grads[index])
         #     grads= torch.cat((grads, torch.flatten(param.grad).cpu()),0)
         # self.grads.append(grads)
-        
+
         self.optim.step()
         # self.train_steps+=1
 
@@ -103,7 +102,8 @@ class node():
         labels = torch.flatten(labels)
         labels = labels.to(self.device, dtype = torch.long)
         # zero the parameter gradients
-        self.model.zero_grad(set_to_none=True)
+        # self.model.zero_grad(set_to_none=True)
+        self.optim.zero_grad()
         # forward + backward + optimize
         outputs = self.model(inputs)
         # optim
@@ -123,7 +123,7 @@ class node():
         labels = labels.to(self.device, dtype = torch.long)
         # zero the parameter gradients
         # model_opt.zero_grad(set_to_none=True)
-        # model_opt.zero_grad()
+        # model_fix.zero_grad(set_to_none=True)
         optimizer.zero_grad()
         # model_2.zero_grad(set_to_none=True)
         # forward + backward + optimize
@@ -134,8 +134,12 @@ class node():
         # loss = ls(outputs, labels)
 
         # loss 2
-        outputs = model_opt(inputs) + model_fix(inputs)
-        loss = self.objective(outputs, labels)
+        reg = 0
+        for p,q in zip(model_opt.parameters(), model_fix.parameters()):
+            reg += torch.norm((p-q),2)
+
+        outputs = model_opt(inputs) + model_fix(inputs) 
+        loss = self.objective(outputs, labels) + 0.005 * reg
 
         # loss 3
         # reg = 0
@@ -150,6 +154,55 @@ class node():
         loss.backward()        
         optimizer.step()
 
+    def train_single_step_con(self, inputs, labels, model_sim, model_all, tmp, mu = 1):
+        inputs = inputs.to(self.device)
+        labels = torch.flatten(labels)
+        labels = labels.to(self.device, dtype = torch.long)
+        # zero the parameter gradients
+        # model_opt.zero_grad(set_to_none=True)
+        self.optim.zero_grad()
+        # forward + backward + optimize
+
+        # contrastive loss
+        output_con_dn = 0
+        # for i in model_all:
+        #     i.to(self.device)
+        #     output_con_dn += torch.exp(F.cosine_similarity(self.intermediate_output(inputs, self.model, self.model.conv2, 'conv2')\
+        #          , self.intermediate_output(inputs, i, i.conv2, 'conv2'), dim = -1)/tmp)
+        # output_con_n = torch.exp(F.cosine_similarity(self.intermediate_output(inputs, self.model, self.model.conv2, 'conv2')\
+        #      , self.intermediate_output(inputs, model_sim, model_sim.conv2, 'conv2'), dim = -1)/tmp)
+        
+        for i in model_all:
+            i.to(self.device)
+            output_con_dn += torch.exp(F.cosine_similarity(self.model(inputs), i(inputs))/tmp)
+        output_con_n = torch.exp(F.cosine_similarity(self.model(inputs), model_sim(inputs))/tmp)
+
+        loss = self.objective(self.model(inputs), labels) - torch.mean(torch.log(output_con_n/output_con_dn)) * mu
+
+        # if self.id == 0:
+        #     # print(self.model.state_dict()['fc1.bias'])
+        #     # print(self.label)
+        #     # for i in range(len(model_all)):
+        #     #     print(i, model_all[i].state_dict()['fc1.bias'])
+        #     print(self.intermediate_output(inputs, self.model, self.model.conv2, 'conv2').shape)
+        #     # print(self.intermediate_output(inputs, model_sim, model_sim.conv2, 'conv2'))
+        #     print(output_con_n, output_con_dn)
+        
+        # self.loss = outputs
+        loss.backward()        
+        self.optim.step()
+
+    def intermediate_output(self, inputs, model, model_layer, layer_name):
+        activation = {}
+        def get_activation(name):
+            def hook(model, input, output):
+                activation[name] = output.detach()
+            return hook
+        
+        model_layer.register_forward_hook(get_activation(layer_name))
+        out = model(inputs)
+        return torch.flatten(activation[layer_name],1)
+
     # for IFCA
     def local_train_loss(self, model):
         model.to(self.device)
